import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.PriorityQueue;
import java.util.StringTokenizer;


/**
 * ICPC - CTU Open Contest 2021
 * Sample Solution: TomTom Cruise
 * 
 * @author Martin Kacer
 */
public class CruiseMK {
	StringTokenizer st = new StringTokenizer("");
	BufferedReader input = new BufferedReader(new InputStreamReader(System.in));
	boolean hasNextToken() {
		try {
			while (!st.hasMoreTokens()) {
				String line = input.readLine();
				if (line == null) return false;
				st = new StringTokenizer(line);
			}
		} catch (IOException ex) { throw new RuntimeException(ex); }
		return true;
	}
	String nextToken() {
		return (!hasNextToken()) ? null : st.nextToken();
	}
	int nextInt() {
		return Integer.parseInt(nextToken());
	}
	public static void main(String[] args) {
		new CruiseMK().run();
	}
	
	static class Edge {
		final int n, w;
		Edge(int nn, int ww) { n = nn; w = ww; }
	}
	static class Elem implements Comparable<Elem> {
		final int n; final long d;
		Elem(int nn, long dd) { n = nn; d = dd; }
		@Override public int compareTo(Elem o) { return Long.compare(this.d, o.d); }
	}
	
	static final long INF = 0x1FFFFFFFFFFFFFFL;
	List<List<Edge>> graph;
	long[] dist;
	int[] start;
	boolean[] closed;
	PriorityQueue<Elem> q = new PriorityQueue<>();
	
	void addEdge(int a, int b, int d) {
		graph.get(a).add(new Edge(b, d));
		graph.get(b).add(new Edge(a, d));
	}
	
	void run() {
		int n = nextInt(), m = nextInt();
		graph = new ArrayList<>(2*n);
		for (int i = 0; i < 2*n; ++i) graph.add(new ArrayList<>());
		dist = new long[2*n]; start = new int[2*n]; closed = new boolean[2*n];
		for (int i = 0; i < n; ++i) addEdge(i, i+n, nextInt());
		for (int i = 0; i < m; ++i) {
			int f = nextInt(), t = nextInt(), w = nextInt();
			addEdge(f, t, w);
		}
		for (int i = 0; i < n; ++i) {
			q.add(new Elem(i+n, 0));
			start[i+n] = i;
			dist[i] = INF;
		}
		long best = INF;
		while (!q.isEmpty()) {
			Elem x = q.poll();
			if (closed[x.n]) continue;
			closed[x.n] = true;
			for (Edge e : graph.get(x.n))
				if (closed[e.n]) {
					if (start[e.n] != start[x.n]) best = Math.min(best, dist[e.n] + dist[x.n] + e.w);
				} else if (dist[e.n] > dist[x.n] + e.w) {
					q.add(new Elem(e.n, dist[e.n] = dist[x.n] + e.w));
					start[e.n] = start[x.n];
				}
		}
		System.out.println(best);
	}
}
